JoinHandle

Struct JoinHandle 

pub struct JoinHandle<T> { /* private fields */ }
Returned by Runtime::spawn to monitor a Task and poll its output.

Implementations

Source

impl<T> JoinHandle<T>

pub fn on_complete<H>(self, complete_hook: H)
where H: OnTaskComplete<T> + 'static,

Set the OnTaskComplete hook that will be called when the task has been complete. If the task was already complete, the hook will be called immediately from the caller’s thread.

pub fn is_active(&self) -> bool

Check if the task is still running

pub fn is_complete(&self) -> bool

Check if the task has been completed successfully

pub fn is_cancelled(&self) -> bool

Check if the task has been cancelled

pub fn join<I: IntoCreateIdle>(self, idle: I) -> Result<T, JoinError>

Join the task, blocking until it is complete or cancelled.

Return Values
  • None - task is still running and an output is not yet available.
  • Ok - task has been completed and the output has been taken. Subsequent calls to take will fail with JoinError::AlreadyTaken.
  • Err - the task has been cancelled or the result has already been taken by Self::try_take.
pub fn try_take(&self) -> Result<Option<T>, JoinError>

Try to take the Task output.

Return Values
  • None - task is still running and an output is not yet available.
  • Ok - task has been completed and the output has been taken. Subsequent calls to take will fail with JoinError::AlreadyTaken.
  • Err - the task has been cancelled or the result has already been taken.
pub fn cancel(&self)

Cancel the current task